/**
 * fis.baidu.com
 */

'use strict';

var spawn = require('child_process').spawn;
var path = require('path');
var fs = require('fs');
var scaffold = require('./lib/scaffold.js');
var ncp = require('ncp');
ncp.limit = 256;

exports.name = 'init';
exports.usage = '<command> [options]';
exports.desc = 'A awesome scaffold of fis';

var templates = require('./config/template.json');

exports.register = function(commander) {
    var o_args = process.argv;
 
    commander
        .option('--verbose', 'output verbose help', Boolean, false)

    fis.util.map(templates, function(key, info){
        commander
            .command(key)
            .description('create a ' + key);
    });

    commander.action(function () {
            var args = Array.prototype.slice.call(arguments);
            var options = args.pop();
            var cmd = args.shift();
            var colors = require('colors');

            if (options.verbose) {
                fis.log.level = fis.log.L_ALL;
                fis.log.throw = true;
            }

            var path = fis.project.getTempPath('init_template', cmd);

            fis.util.del(path);

            var conf = templates[cmd];

            if (!conf){
                fis.log.error('invalid init command, see -h');
            }

            fis.log.notice('downloading and unzipping');
            fis.util.download(conf.path, function(){
                var dir = process.cwd();

                check_dir(dir);

                scaffold.prompt(path, {
                },function () {
                    fis.log.notice('moving files');
                    ncp(path + '/', dir, function (err) {
                        if (err) {
                           return fis.log.error(err);
                        }
                        fis.log.notice('done');
                    });
                });
            }, path);
        });
};

function check_dir(dir) {
    dir = path.resolve(dir);
    if (fis.util.fs.existsSync(dir) && process.cwd() != dir) {
        fis.log.error('the directory has already exist, the process will stop.');        
    }
}