/**
 * fis.baidu.com
 */

var fs = require('fs');

module.exports = {
    prompt: function (dir, options, cb) {
        
        if (fis.util.is(options, 'Function')) {
            cb = options;
            options = {};
        }

        var keyword_reg = options && options.keyword_reg || /\{\{-([\s\S]*?)-\}\}/ig;
        var files = fis.util.find(dir);
        var prompts = {};
        var cache = {};
        var promptDirs = {};
        var preset = options.preset || {};

        fis.util.map(files, function (index, filepath) {
            var content = fis.util.fs.readFileSync(filepath, {
                encoding: 'utf8'
            });
            content.replace(keyword_reg, function(m, $1) {
                if (prompts[$1]) {
                    prompts[$1].push(filepath);
                } else {
                    prompts[$1] = [filepath];
                }
                return m;
            });
            cache[filepath] = content;
        });

        var dirs = readdirSync(dir);

        dirs.forEach(function(dir){
            dir.replace(keyword_reg, function(m, $1) {
                if (!prompts[$1]) {
                    prompts[$1] = [];
                }
                promptDirs[dir] = true;
                return m;
            });
        });

        var prompt_handle = require('prompt');
        var opts = [];
        fis.util.map(prompts, function (k, v) {
            if (preset[k]){
                return false;
            };
            opts.push({
                name: k,
                required: true
            });
        });

        prompt_handle.start();
        prompt_handle.get(opts, function (err, result) {
            fis.util.merge(result, options.preset);
            fis.util.map(result, function (k, v) {
                var process_files = prompts[k];
                fis.util.map(process_files, function (index, filepath) {
                    var con = cache[filepath].replace(keyword_reg, function (m, $1) {
                        if ($1 == k) {
                            m = v;
                        }
                        return m;
                    });

                    fis.util.fs.writeFileSync(filepath, con);
                    
                    cache[filepath] = con;
                });
                dirs.reverse().forEach(function(dir){
                    if(promptDirs[dir]){
                        var newDir = dir.replace(keyword_reg, function(m, $1) {
                            if ($1 == k) {
                                m = v;
                            }
                            return m;
                        });
                        fis.util.copy(dir, newDir);
                        fis.util.del(dir);
                    }
                });
            });
            cb && cb();
        });
    },
    
    mv: function (source, dist, include, exclude) {
        fis.util.copy(source, dist, include, exclude);
    },
};


function values(o) {
    var vals = [];
    for (var i in o) {
        if (o.hasOwnProperty(i)) {
            vals.push(o[i]);
        }
    }
    return vals;
}

function readdirSync(dir) {
    var results = [];
    var list = fs.readdirSync(dir);
    list.forEach(function(file) {
        file = dir + '/' + file;
        var stat = fs.statSync(file);
        if (stat && stat.isDirectory()) {
            results.push(file)
        }
    });
    return results
};